//+-------------------------------------------------------------------------------------+
//|                                                                FiboPivot_Expert.mq4 |
//|                                                                           Scriptong |
//|                                                                                     |
//+-------------------------------------------------------------------------------------+
#property copyright "Scriptong"
#property link      "scriptong@mail.ru"

//---- input parameters
extern double    Lots        = 0.1;                          //   
extern string    A1 = "     ";
extern double    Resistance3=138.2;                        //   
extern double    Resistance2=100.0;                              //  
extern double    Resistance1=61.8;                          //   
extern double    Support1=61.8;                                //   
extern double    Support2=100.0;                                     //  
extern double    Support3=138.2;                                //   
extern string    A2 = "=====================================";
extern string    OpenOrderSound = "ok.wav";                  //    
extern int       MagicNumber = 17589 ;                         //   
bool Activate, FreeMarginAlert, FatalError;
double Tick, Spread, StopLevel, MinLot, MaxLot, LotStep, Res1, Res2, Res3, Supp1, Supp2,
       Supp3, Pivot;
datetime LastSell, LastBuy;
int TypeOrder, LastType, TicketOrder;


//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
int init()
  {
//----
   Activate = False;
// - 1 - ==      ========================================
   Tick = MarketInfo(Symbol(), MODE_TICKSIZE);                         //  
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                  //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //   
   MinLot = MarketInfo(Symbol(), MODE_MINLOT);    //    
   MaxLot = MarketInfo(Symbol(), MODE_MAXLOT);   //    
   LotStep = MarketInfo(Symbol(), MODE_LOTSTEP);          //    
// - 1 - ==   =============================================================

// - 2 - ==          =======
   Lots = MathRound(Lots/LotStep)*LotStep; //     
   if(Lots < MinLot || Lots > MaxLot) //     MinLot    MaxLot
     {
      Comment(" Lots     !  !");
      return(0);
     }
// - 2 - ==   =============================================================

// - 3 - ==    ==============================================
   Comment(",    ...");
   int P = 0;
   while(P < 120) 
    {
     datetime NowDay = iTime(Symbol(), PERIOD_D1, 0);
     if (GetLastError() != 4066) break;
     Sleep(1000);
     P++;
    }
   Comment(""); 
// - 3 - ==   =============================================================

// - 4 - ==         ==========================
   LastSell = 0;
   LastBuy = 0;
   for (int i = 0; i < OrdersTotal(); i++)
     if (OrderSelect(i, SELECT_BY_POS))
       if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
         {
          if (OrderType() == OP_BUY && OrderOpenTime() >= NowDay)
            LastBuy = NowDay;
          if (OrderType() == OP_SELL && OrderOpenTime() >= NowDay)
            LastSell = NowDay;
         } 
   for (i = 0; i < OrdersHistoryTotal(); i++)
     if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))
       if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
         {
          if (OrderType() == OP_BUY && OrderOpenTime() >= NowDay)
            LastBuy = NowDay;
          if (OrderType() == OP_SELL && OrderOpenTime() >= NowDay)
            LastSell = NowDay;
         } 
// - 4 - ==   =============================================================


   Activate = True; //    ,    
   
//----
   return(0);
  }

//+-------------------------------------------------------------------------------------+
//|                                                       |
//+-------------------------------------------------------------------------------------+
int deinit()
{
//----
   Comment("");
//----
   return(0);
}

//+-------------------------------------------------------------------------------------+
//|                                              |
//+-------------------------------------------------------------------------------------+
double ND(double A)
{
 return(NormalizeDouble(A, Digits));
}  
 
//+-------------------------------------------------------------------------------------+
//|                                                         |
//+-------------------------------------------------------------------------------------+
string ErrorToString(int Error)
{
 switch(Error)
   {
    case 2: return("  ,   ."); 
    case 5: return("    ,  ."); 
    case 6: return("   ,   ."); 
    case 64: return(" ,   .");
    case 132: return(" ."); 
    case 133: return(" ."); 
    case 149: return(" ."); 
   }
}
  
//+-------------------------------------------------------------------------------------+
//|                                                                      |
//| :                                                                         |
//|   True -                                                       |
//|   False -                                                             |
//+-------------------------------------------------------------------------------------+
bool OpenOrder(int Type, double Price, double SL, double TP)
{
 //     
 if(AccountFreeMarginCheck(Symbol(), OP_BUY, Lots) <= 0 || GetLastError() == 134) 
  {
   if(!FreeMarginAlert)
    {
     Print("    . Free Margin = ", 
           AccountFreeMargin());
     FreeMarginAlert = True;
    } 
   return(False);  
  }
 FreeMarginAlert = False;  
 // ---------------------------------------------

 switch (Type)
   {
    case OP_BUY: string S = "BUY"; break;
    case OP_SELL: S = "SELL"; break;
    case OP_BUYSTOP: S = "BUYSTOP"; break;
    case OP_SELLSTOP: S = "SELLSTOP"; break;
    case OP_BUYLIMIT: S = "BUYLIMIT"; break;
    case OP_SELLLIMIT: S = "SELLLIMIT"; break;
   }

 if(WaitForTradeContext())  //    
   {  
    Comment("     ", S, " ...");
    int ticket=OrderSend(Symbol(), Type, Lots, Price, 0, 
               SL, TP, NULL, MagicNumber, 0, CLR_NONE);        //  
    //     
    if(ticket<0)
      {
       int Error = GetLastError();
       if(Error == 2 || Error == 5 || Error == 6 || Error == 64 
          || Error == 132 || Error == 133 || Error == 149)     //   
         {
          Comment("     . . "+
                   ErrorToString(Error)+"  !");
          FatalError = True;
         }
        else 
         Comment("   ", S, ": ", Error);       //  
       return(False);
      }
    // ---------------------------------------------
    
    //      
    Comment(" ", S, "  !"); 
    PlaySound(OpenOrderSound); 
    return(True); 
    // ------------------------
   }
  else
   {
    Comment("     !");
    return(False);  
   } 
}

//+-------------------------------------------------------------------------------------+
//|   .   ,   True,  - False    |
//+-------------------------------------------------------------------------------------+
bool WaitForTradeContext()
{
 int P = 0;
 //  ""
 while(IsTradeContextBusy() && P < 5)
   {
    P++;
    Sleep(1000);
   }
 // -------------  
 if(P == 5)
   return(False);
 return(True);    
}

//+-------------------------------------------------------------------------------------+
//|   /                                              |
//+-------------------------------------------------------------------------------------+
void GetSignal()
{
// - 1 - ==         ===================
 double HighDay = iHigh(Symbol(), PERIOD_D1, 1);              //   
 double LowDay = iLow(Symbol(), PERIOD_D1, 1);                 //   
 double CloseDay = iClose(Symbol(), PERIOD_D1, 1);            //   
 double Width = HighDay - LowDay;                         //   
 Pivot = (HighDay+LowDay+CloseDay)/3;             // Pivot -   
 Res1 = Pivot + Width*(Resistance1/100.0);                        //  
 Res2 = Pivot + Width*(Resistance2/100.0);                        //  
 Res3 = Pivot + Width*(Resistance3/100.0);                        //  
 Supp1 = Pivot - Width*(Support1/100.0);                              //  
 Supp2 = Pivot - Width*(Support2/100.0);                              //  
 Supp3 = Pivot - Width*(Support3/100.0);                              //  
// - 1 - ==   =============================================================
}  

//+-------------------------------------------------------------------------------------+
//|   .    ,   False.                |
//|    True.                                                               |
//+-------------------------------------------------------------------------------------+
bool CloseAll()
{
 for (int i = OrdersTotal()-1; i >= 0; i--)
   if (OrderSelect(i, SELECT_BY_POS))
     if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
       if (OrderOpenTime() < iTime(Symbol(), PERIOD_D1, 0))
         if (WaitForTradeContext())  
           {
            if (OrderType() < 2)                                    //   
              {
               RefreshRates();
               if (OrderType() == OP_BUY)
                 double Pr = ND(Bid);
                else
                 Pr = ND(Ask);
               if (!OrderClose(OrderTicket(), OrderLots(), Pr, 3)) //   
                return(False);                       //    -  False
              }  
             else
              if (!OrderDelete(OrderTicket()))
                return(False); 
           }   
          else
           return(False); // False,      

 return(True);                                                //   ,   0
}  


//+-------------------------------------------------------------------------------------+
//|        ,   |
//+-------------------------------------------------------------------------------------+
void PartiallyClose()
{
 for (int i = OrdersTotal()-1; i >= 0; i--)
   if (OrderSelect(i, SELECT_BY_POS))
     if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber 
        && OrderType() < 2)                                  //   
        {
         if (StringFind(OrderComment(), "from #", 0) < 0)
           {                  //       
            if ((OrderType() == OP_BUY && ND(Bid) >= ND(Res2)) ||
                (OrderType() == OP_SELL && ND(Bid) <= ND(Supp2)))
              {                                          //   
               double LotsClose = MathMax(MinLot,           //   
                                          MathRound((OrderLots()/2)/LotStep)*LotStep);
               if (WaitForTradeContext())                           
                 if (!OrderClose(OrderTicket(), LotsClose, IF(OrderType() == OP_BUY, 
                                 ND(Bid), ND(Ask)), 3))                           
                   return;              
              }  
           }
          else
           {                                                 //    
            bool Res = False;
            if (OrderType() == OP_BUY)
              if (ND(OrderStopLoss()) < ND(OrderOpenPrice()) &&
                  ND(Bid - OrderOpenPrice()) > StopLevel)
                Res = True;  
            if (OrderType() == OP_SELL)
              if (ND(OrderStopLoss()) > ND(OrderOpenPrice()) &&
                  ND(OrderOpenPrice() - Ask) > StopLevel)
                Res = True;  
            if (Res)    
              if (WaitForTradeContext())
                if (!OrderModify(OrderTicket(), 0, ND(OrderOpenPrice()), 
                                 OrderTakeProfit(), 0))
                return;               
           } 
        }
}

//+-------------------------------------------------------------------------------------+
//|    "if-else"                                                 |
//+-------------------------------------------------------------------------------------+
double IF(bool Condition, double IfTrue, double IfFalse)
{
 if (Condition) return(IfTrue);    //   ,    IfTrue
 else           return(IfFalse);    //   ,    IfFalse
}  

//+-------------------------------------------------------------------------------------+
//|  START                                                               |
//+-------------------------------------------------------------------------------------+
int start()
  {
// - 1 -  ==    ? ===========================================
   if (!Activate || FatalError)             //   ,  
    return(0);           //  init         
// - 1 -  ==   ============================================================

// - 2 - ==         =======================
   PartiallyClose();
// - 2 -  ==   ============================================================

// - 3 -  ==     ===============================================
   datetime NowDay = iTime(Symbol(), PERIOD_D1, 0);
   if (LastBuy == NowDay && LastSell == NowDay)  //  ,   
     return(0);                                                               // 
// - 3 -  ==   ============================================================

// - 4 - ==      ========================================
   Spread = ND(MarketInfo(Symbol(), MODE_SPREAD)*Point);                  //  
   StopLevel = ND(MarketInfo(Symbol(), MODE_STOPLEVEL)*Point);  //   
// - 4 -  ==   ============================================================
     
// - 5 - ==    =====================================================
   GetSignal();
// - 5 -  ==   ============================================================
   
// - 6 - ==   BuyStop ====================================================
   double Price = ND(Res1 + Spread);
   bool Res = CloseAll();                     //       
   RefreshRates();
   if (Res && Ask < Price && LastBuy < NowDay)     //       
     {                                                   //    
      double TP = ND(IF(Res3 - Price > StopLevel, Res3, Price + StopLevel));
      double SL = ND(IF(Price - Pivot - Tick > StopLevel+Spread, Pivot - Tick,
                        Price - StopLevel - Spread));
      if (ND(Price - Ask) <= StopLevel)        //    ,  
        return(0);                                                     //  
      if (!OpenOrder(OP_BUYSTOP, Price, SL, TP))      //     
        return(0);                                     //     
      LastBuy = NowDay;  
     }
   if (!Res) return(0);           //    ,    
// - 6 -  ==   ============================================================
     
// - 7 - ==   SellStop ===================================================
   Price = ND(Supp1);
   if (Bid > Price && LastSell < NowDay)   //        
     {                                                                //  
      TP = ND(IF(Price - Supp3 + Spread > StopLevel, Supp3 + Spread, 
                 Price - StopLevel));
      SL = ND(IF(Pivot + Tick - Price > StopLevel, Pivot + Spread + Tick,
                        Price + StopLevel + Spread));
      if (ND(Bid - Price) <= StopLevel)        //    ,  
         return(0);                                                    //  
      if (!OpenOrder(OP_SELLSTOP, Price, SL, TP))     //     
        return(0);                                     //     
      LastSell = NowDay;
     }
// - 7 -  ==   ============================================================
   return(0);
  }
//+------------------------------------------------------------------+